searchData={"items":[{"type":"module","title":"LiveSync","doc":"LiveSync is the core of all interactions include a postgres replication process and LiveView hooks to automatically sync data to the client.","ref":"LiveSync.html"},{"type":"module","title":"Installation - LiveSync","doc":"This project builds on top of PostgreSQL replication and it requires PostgreSQL 14+. You must also enable replication in your PostgreSQL instance:\n\n    ALTER SYSTEM SET wal_level='logical';\n    ALTER SYSTEM SET max_wal_senders='64';\n    ALTER SYSTEM SET max_replication_slots='64';\n\nThen **you MUST restart your database**.\n\nAdd `live_sync` to the list of dependencies in `mix.exs`:\n\n          def deps do\n            [\n              {:live_sync, \"~> 0.1.0\"}\n            ]\n          end\n\nAdd a migration to setup replication (requires superuser permissions to subscribe to all tables):\n\n          defmodule MyApp.Repo.Migrations.SetupLiveSync do\n            use Ecto.Migration\n\n            def up do\n              LiveSync.Migration.up()\n              # If you don't have superuser you can pass specific tables\n              # LiveSync.Migration.up([\"table1\", \"table2\"])\n            end\n\n            def down do\n              LiveSync.Migration.down()\n            end\n          end\n\nAdd `LiveSync` to your supervision tree:\n\n* `repo` (required): The Ecto repo to use for replication.\n\n* `otp_app` (required): The OTP app to use to lookup schemas deriving the watch protocol.\n\n        defmodule MyApp.Application do\n          @moduledoc false\n\n          use Application\n\n          @impl true\n          def start(_type, _args) do\n            children = [\n              ...\n              {LiveSync, [repo: MyApp.Repo, otp_app: :my_app]}\n            ]\n\n            opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n            Supervisor.start_link(children, opts)\n          end\n        end","ref":"LiveSync.html#module-installation"},{"type":"module","title":"Usage - LiveSync","doc":"For any Ecto schemas you want to watch, add the `LiveSync.Watch` derive:\n\n  * `id` (optional): The primary key on the schema, defaults to `:id`.\n\n  * `subscription_key` (required): The field on the schema that is used to filter messages before sending to the client.\n\n  * `table` (optional): The table to watch, defaults to the schema's table name, if using a view you need to specify the table name\n\n        defmodule MyApp.MyObject do\n          use Ecto.Schema\n\n          @derive {LiveSync.Watch,\n                    [\n                      subscription_key: :organization_id,\n                      table: \"objects\"\n                    ]}\n\n          schema \"visible_objects\" do\n            field :name, :string\n            field :organization_id, :integer\n          end\n\n          ...\n        end\n\nAdd the `LiveSync` macro to any LiveView module you want to automatically sync data:\n\n  * `subscription_key` (required): This is the key that MUST exist in the assigns of the LiveView and is used for the subscription. This value must match what is in the schema's `@derive` attribute.\n\n  * `watch` (required): A list of keys in assigns that should be watched. You may optionally specify a tuple with options. Schema is required for lists of objects to support inserting.\n\n        use LiveSync,\n          subscription_key: :organization_id,\n          watch: [\n            :single_object,\n            list_of_objects: [schema: MyApp.MyObject]\n          ]","ref":"LiveSync.html#module-usage"},{"type":"module","title":"Handling Sync Events - LiveSync","doc":"An optional callback can also be added to the LiveView module to handle the updated data. This callback will be called for each assign key that is watched and changed. It must return the socket with updated assigns.\n\n    def sync(:list_of_objects, updated, socket) do\n      updates =\n        updated\n        |> Enum.filter(&is_nil(&1.executed_at))\n        |> Enum.sort_by(& &1.name)\n        |> Repo.preload([...])\n\n      assign(socket, list_of_objects: updates)\n    end","ref":"LiveSync.html#module-handling-sync-events"},{"type":"function","title":"LiveSync.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"LiveSync.html#child_spec/1"},{"type":"function","title":"LiveSync.lookup_info/1","doc":"Returns the info of how to compare the given struct for updates.\n\n    iex> LiveSync.lookup_info(%MyObject{id: 1, organization_id: 2})\n    {MyObject, 1}","ref":"LiveSync.html#lookup_info/1"},{"type":"function","title":"LiveSync.opts/1","doc":"","ref":"LiveSync.html#opts/1"},{"type":"function","title":"LiveSync.start_link/1","doc":"","ref":"LiveSync.html#start_link/1"},{"type":"function","title":"LiveSync.subscription_key/1","doc":"Returns the value for the subscription key for the given struct\n\n    iex> LiveSync.subscription_key(%MyObject{id: 1, organization_id: 2})\n    2","ref":"LiveSync.html#subscription_key/1"},{"type":"protocol","title":"LiveSync.Watch","doc":"Protocol for defining which Ecto schemas to watch for changes.\n\nTwo options are supported:\n- subscription_key (required): The field on the schema that is used to filter messages that a LiveView session is subscribed to\n- table (optional): The table to watch, defaults to the schema's table name, if using a view you need to specify the table name\n\nExample:\n\n    @derive {LiveSync.Watch,\n            [\n              subscription_key: :organization_id,\n              table: \"table\"\n            ]}","ref":"LiveSync.Watch.html"},{"type":"function","title":"LiveSync.Watch.info/1","doc":"","ref":"LiveSync.Watch.html#info/1"},{"type":"function","title":"LiveSync.Watch.subscription_key/1","doc":"","ref":"LiveSync.Watch.html#subscription_key/1"},{"type":"type","title":"LiveSync.Watch.t/0","doc":"All the types that implement this protocol.","ref":"LiveSync.Watch.html#t:t/0"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.36.1"}}